﻿using System;
using System.Data.Entity;
using System.Data.Entity.Core.Objects;
using System.Data.Entity.Infrastructure;
using System.Linq;
using EFExplicitLoading.Models;

namespace EFExplicitLoading {

    internal class Program {
        private static void Main(string[] args) {
            //添加测试数据
            using (var context = new ExplicitContext()) {
                //先清空数据
                context.Database.ExecuteSqlCommand("delete from product");
                context.Database.ExecuteSqlCommand("delete from brand");
                context.Database.ExecuteSqlCommand("delete from category");

                var category1 = new Category {Name = "服装"};
                var category2 = new Category {Name = "家电"};
                var brand1 = new Brand {Name = "品牌1"};
                var brand2 = new Brand {Name = "品牌2"};

                context.Products.Add(new Product {Brand = brand1, Category = category1, Title = "产品1"});
                context.Products.Add(new Product {Brand = brand1, Category = category2, Title = "产品2"});
                context.Products.Add(new Product {Brand = brand1, Category = category2, Title = "产品3"});
                context.Products.Add(new Product {Brand = brand2, Category = category2, Title = "产品4"});
                context.Products.Add(new Product {Brand = brand2, Category = category2, Title = "产品5"});
                context.Products.Add(new Product {Brand = brand2, Category = category2, Title = "产品6"});
                context.SaveChanges();
            }


            using (var context = new ExplicitContext()) {
                //禁用懒加载
                context.Configuration.LazyLoadingEnabled = false;

                var category = context.Categories.First(c => c.Name == "家电");

                //判断分类关联的产品是否已经被加载
                if (!context.Entry(category).Collection(x => x.Products).IsLoaded) {
                    context.Entry(category).Collection(x => x.Products).Load();
                    Console.WriteLine("分类{0}的产品被显示加载...", category.Name);
                }

                Console.Write("分类{0}下的共有{1}件产品", category.Name, category.Products.Count());

                foreach (var product in context.Products) {
                    if (!context.Entry(product).Reference(x => x.Category).IsLoaded) {
                        context.Entry(product).Reference(x => x.Category).Load();
                        Console.WriteLine("分类{0}被显示加载.", product.Category.Name);
                    }
                    else {
                        Console.WriteLine("分类{0}已经加载过了.", product.Category.Name);
                    }
                }

                //重新使用category计算 产品数量
                Console.WriteLine("产品加载完后，分类{0}下有{1}件产品", category.Name, category.Products.Count());

                category.Products.Clear();
                Console.WriteLine("产品集合被清空了.");
                Console.WriteLine("此时分类{0}下有{1}件产品", category.Name, category.Products.Count());


                context.Entry(category).Collection(x => x.Products).Load();
                Console.WriteLine("重新显示加载产品");
                Console.WriteLine("此时分类{0}下有{1}件产品", category.Name, category.Products.Count());

                //使用ObjectContext刷新实体
                var objectContext = ((IObjectContextAdapter) context).ObjectContext;
                var objectSet = objectContext.CreateObjectSet<Product>();
                objectSet.MergeOption = MergeOption.OverwriteChanges;
                objectSet.Load();
                //MergeOption.AppendOnly
                //MergeOption.OverwriteChanges
                //MergeOption.NoTracking
                //MergeOption.PreserveChanges
                

                Console.WriteLine("产品集合以MergeOption.OverwriteChanges的方式重新加载");
                Console.WriteLine("此时分类{0}下有{1}件产品", category.Name, category.Products.Count());
            }


            Console.WriteLine("测试部分加载...");
            //测试部分加载，然后加载所有
            using (var context = new ExplicitContext()) {
                //禁用懒加载
                context.Configuration.LazyLoadingEnabled = false;
                var category = context.Categories.First(c => c.Name == "家电");
                //先加载1条数据
                Console.WriteLine("先加载1条数据");
                context.Entry(category).Collection(x => x.Products).Query().Take(1).Load();
                Console.WriteLine("分类{0}下共有{1}件产品", category.Name, category.Products.Count());

                //加载所有
                context.Entry(category).Collection(x => x.Products).Load();
                Console.WriteLine("加载所有数据");

                Console.WriteLine("此时分类{0}下有{1}件产品", category.Name, category.Products.Count());
            }

            Console.ReadKey();
        }
    }

}